import numpy as np


class CollectionService:
    def __init__(self, api):
        self.api = api

    def _validate_collection_args(self, name, distance, dimension):
        if not (1 <= len(name) <= 36):
            return "Collection name must be 1–36 characters"

        VALID_DISTANCES = {"cosine", "dot", "l2"}
        if distance.lower() not in VALID_DISTANCES:
            return "Distance must be cosine, dot, or l2"

        if dimension <= 0 or dimension > 9999:
            return "Dimension must be between 1 and 9999"

        return None

    def _normalize_vectors(self, vectors):
        """
        Accepts:
        - List[List[float]]
        - np.ndarray (2D)


        Returns:
        - List[List[float]]
        """

        if isinstance(vectors, np.ndarray):
            if vectors.ndim != 2:
                raise ValueError("Vectors ndarray must be 2-dimensional")
            return vectors.tolist()

        if isinstance(vectors, list):
            if not all(isinstance(v, list) for v in vectors):
                raise TypeError("Vectors must be a list of lists")
            return vectors

        raise TypeError("Vectors must be a list of lists or a numpy ndarray")

    def create_collection(self, collection_name, distance, dimension=100):

        error = self._validate_collection_args(collection_name, distance, dimension)

        if error:
            return {"error": error}

        existing = self.api.get_collection(collection_name).get("result")

        if existing:
            return {"error": "Collection already exists"}

        payload = {
            "collection_name": collection_name,
            "dimension": dimension,
            "distance": distance.lower(),
        }

        return self.api.create_collection(payload)

    def recreate_collection(self, collection_name, distance, dimension=100):

        error = self._validate_collection_args(collection_name, distance, dimension)

        if error:
            return {"error": error}

        existing = self.api.get_collection(collection_name).get("result")
        if existing:
            self.api.delete_collection(collection_name)

        payload = {
            "collection_name": collection_name,
            "dimension": dimension,
            "distance": distance.lower(),
        }

        return self.api.create_collection(payload)

    def delete_collection(self, collection_name):

        existing = self.api.get_collection(collection_name).get("result")
        if not existing:
            return {"error": "Collection not found"}

        return self.api.delete_collection(collection_name)

    def list_collection(self):
        # list collections
        return self.api.list_collections()

    def upsert_points(self, collection_name, vectors, ids, payloads):

        collection = self.api.get_collection(collection_name).get("result")
        if not collection:
            return {"error": "Collection not found"}

        dim = int(collection["dimension"])

        try:
            vectors = self._normalize_vectors(vectors)
        except (TypeError, ValueError) as e:
            return {"error": str(e)}

        if any(len(v) != dim for v in vectors):
            return {"error": f"Vector dimension must be {dim}"}

        if not (len(vectors) == len(ids) == len(payloads)):
            return {"error": "Vectors, ids and payload size mismatch"}

        body = {
            "vectors": vectors,
            "ids": ids,
            "payload": payloads,
        }

        return self.api.upsert_points(collection_name, body)

    def delete_points(self, collection_name, ids):
        collection = self.api.get_collection(collection_name).get("result")
        if not collection:
            return {"error": "Collection not found"}

        if not ids:
            return {"error": "IDs required"}

        return self.api.delete_points(collection_name, {"ids": ids})

    def search_query(self, collection_name, query_vector, limit=1):

        collection = self.api.get_collection(collection_name).get("result")
        if not collection:
            return {"error": "Collection not found"}

        #  Normalize numpy arrays
        if isinstance(query_vector, np.ndarray):
            query_vector = query_vector.tolist()

        # Validate type after normalization
        if not isinstance(query_vector, list):
            return {"error": "query_vector must be a list or numpy array"}

        if len(query_vector) != int(collection["dimension"]):
            return {"error": "Vector dimension mismatch"}

        return self.api.search_points(
            collection_name,
            {"vectors": query_vector, "limit": limit},
        )



from enum import Enum


class Distance(str, Enum):
    DOT = "Dot"
    COSINE = "Cosine"
    L2 = "L2"


class VectorConfig:
    def __init__(self, size=10, distance=""):
        self.dimension = size
        self.distance = distance



from zoro_client import ZoroClient, VectorConfig, Distance


client = ZoroClient(host="localhost", port=6464)

def test_create_collection():
    client.delete_collection("test")
    response = client.create_collection(collection_name="test", vector_config=VectorConfig(size=4, distance=Distance.COSINE))
    assert "result" in response
    assert response["result"]["collection_name"] == "test"
    assert response["result"]["dimension"] == 4
    assert response["result"]["distance"] == "cosine"


def test_delete_collection():
    client.recreate_collection(collection_name="test", vector_config=VectorConfig(size=4, distance=Distance.COSINE))
    response = client.delete_collection("test")
    assert "result" in response
    assert response["result"]["collection_name"] == "test"


def test_list_collections_basic():
    response = client.list_collections()

    assert "result" in response
    assert isinstance(response["result"], list)




import requests


class HTTPClient:
    def __init__(self, base_url: str):
        self.base_url = base_url.rstrip("/")

    def _handle_response(self, response: requests.Response):
        try:
            return response.json()
        except ValueError:
            raise RuntimeError(
                f"Non-JSON response ({response.status_code}): {response.text}"
            )

    def get(self, endpoint: str):
        url = f"{self.base_url}/{endpoint.lstrip('/')}"

        resp = requests.get(url)
        return self._handle_response(resp)

    def post(self, endpoint: str, body=None):
        url = f"{self.base_url}/{endpoint.lstrip('/')}"

        resp = requests.post(url, json=body or {})

        return self._handle_response(resp)

    def delete(self, endpoint: str, body=None):
        url = f"{self.base_url}/{endpoint.lstrip('/')}"

        resp = requests.delete(url, json=body or {})
        return self._handle_response(resp)



class APIService:
    def __init__(self, client):
        self.client = client

    # Collections
    def get_collection(self, name):
        return self.client.get(f"/collections/{name}")

    def list_collections(self):
        return self.client.get("/collections/")

    def create_collection(self, payload):
        return self.client.post("/collections/", payload)

    def delete_collection(self, name):
        return self.client.delete(f"/collections/{name}")

    # Points
    def upsert_points(self, collection_name, payload):
        return self.client.post(f"/collections/{collection_name}/points", payload)

    def delete_points(self, collection_name, payload):
        return self.client.delete(f"/collections/{collection_name}/points", payload)

    def search_points(self, collection_name, payload):
        return self.client.post(
            f"/collections/{collection_name}/points/search", payload
        )



import requests
from .http_client import HTTPClient
from .api import APIService
from .service import CollectionService


class ZoroClient:
    def __init__(self, host=None, port=None, url=None):
        self.port = port
        self.host = host
        self.url = url
        if url is None:
            if host is None:
                self.host = "localhost"
            if port is None:
                self.port = 6464
            self.url = f"http://{host}:{port}"

        if not self._is_server_reachable(self.url):
            raise ConnectionError(f"Unable to connect to server at {self.url}")

        client = HTTPClient(self.url)
        api = APIService(client)
        self.collection = CollectionService(api)

    def _is_server_reachable(self, url):
        # Helper method to check if the server is reachable.
        try:
            response = requests.head(url, timeout=5)
            return response.status_code // 100 == 2
        except requests.exceptions.RequestException:
            return False

    def create_collection(self, collection_name, vector_config):
        dimension = vector_config.dimension
        distance = vector_config.distance
        return self.collection.create_collection(
            collection_name=collection_name,
            dimension=dimension,
            distance=distance,
        )

    def recreate_collection(self, collection_name, vector_config):
        dimension = vector_config.dimension
        distance = vector_config.distance
        return self.collection.recreate_collection(
            collection_name=collection_name,
            dimension=dimension,
            distance=distance,
        )

    def delete_collection(self, collection_name):
        return self.collection.delete_collection(
            collection_name=collection_name,
        )

    def list_collections(self):
        """Add filter by dimension, distance, name starting with and name ending with ex. `%ies` or `mov%`"""
        return self.collection.list_collection()

    def upsert_points(self, collection_name, vectors, ids, payloads):

        return self.collection.upsert_points(
            collection_name=collection_name,
            vectors=vectors,
            ids=ids,
            payloads=payloads,
        )

    def delete_points(self, collection_name, ids):
        return self.collection.delete_points(collection_name=collection_name, ids=ids)

    def search(self, collection_name, query_vector, limit=1):
        return self.collection.search_query(
            collection_name=collection_name,
            query_vector=query_vector,
            limit=limit,
        )
